#!/usr/bin/perl  -w

#main program
#This program is invoked in each subject's directory

$data_file_name = "data.tex";

%subjects = ();
$subjects{"IntStack"} = 0;
$subjects{"uniqueBoundedStack"} = 1;
$subjects{"ShoppingCart"} = 2;
$subjects{"BankAccount"} = 3;
$subjects{"BinarySearchTree"} = 4;
$subjects{"BinomialHeap"} = 5;
$subjects{"DisjSet"} = 6;
$subjects{"FibonacciHeap"} = 7;
$subjects{"HashMap"} = 8;
$subjects{"LinkedList"} = 9;
$subjects{"TreeMap"} = 10;

%subjects_show_name = ();
$subjects_show_name{"IntStack"} = "IntStack";
$subjects_show_name{"uniqueBoundedStack"} = "UBStack";
$subjects_show_name{"ShoppingCart"} = "ShoppingCart";
$subjects_show_name{"BankAccount"} = "BankAccount";
$subjects_show_name{"BinarySearchTree"} = "BinSearchTree";
$subjects_show_name{"BinomialHeap"} = "BinomialHeap";
$subjects_show_name{"DisjSet"} = "DisjSet";
$subjects_show_name{"FibonacciHeap"} = "FibonacciHeap";
$subjects_show_name{"HashMap"} = "HashMap";
$subjects_show_name{"LinkedList"} = "LinkedList";
$subjects_show_name{"TreeMap"} = "TreeMap";

%frontends = ();
$frontends{"pairwiseEquals"} = 5;
$frontends{"monitorEquals"} = 4;
$frontends{"traverseField"} = 3;
$frontends{"monitorWField"} = 2;
$frontends{"monitorSMSeq"} = 1;
$frontends{"monitorSeq"} = 0;

%frontends_show_name = ();
$frontends_show_name{"pairwiseEquals"} = "PairwiseEquals";
$frontends_show_name{"monitorEquals"} = "MonitorEquals";
#$frontends_show_name{"monitorWField"} = "WholeStateW";
$frontends_show_name{"traverseField"} = "WholeState";
$frontends_show_name{"monitorSMSeq"} = "ModifyingSeq";
$frontends_show_name{"monitorSeq"} = "WholeSeq";

$tools{"jcrasher"} = 0;
$tools{"jtest"} = 1;

$iterations{"Orig"} = 0;
$iterations{"IterationOne"} = 1;
$iterations{"IterationTwo"} = 2;

$iterations_show_name{"Orig"} = "Existing";
$iterations_show_name{"IterationOne"} = "Iteration 1";
$iterations_show_name{"IterationTwo"} = "Iteration 2";

$frontend_alternative = "pairwiseEquals|monitorEquals|monitorWField|traverseField|monitorSMSeq|monitorSeq";
$tool_alternative = "JCrasher|Jtest";
$iteration_alternative = "IterationOne|IterationTwo|Orig";

%testMinData = ();
%testGenData = ();

foreach $subject (keys %subjects) {
	foreach $tool (keys %tools) {
		open(DATA, $subject."/".$tool."/".$data_file_name) || die "opening data tex file ".$subject."\\".$data_file_name." $!";
		while(<DATA>) {
			$line = $_;
			chomp($line);	  
			if ($line =~ /^\\newcommand\{\\testMin(.+)($frontend_alternative)($tool_alternative)(.+)\} \{(.*)\}$/) {
				#\newcommand{\testMinIntStackmonitorEqualsJCrasherTime} {1.01}
				$subject_name = $1;
				$frontend_name = $2;
				$tool_name = $3;
				$measured_item = $4;
				$measured_result = $5;
				$measured_result =~ s/\*//;
				$new_measured_result = $measured_result;
				$new_measured_result =~ s/\\%//;
				if ($new_measured_result ne $measured_result) {
					$new_measured_result = $new_measured_result/100.0;
				}				
				$testMinData{$measured_item}{$tool_name}{$frontend_name}{$subject_name} = $new_measured_result;			
			} elsif ($line =~ /^\\newcommand\{\\testGen(.+)($frontend_alternative)($tool_alternative)($iteration_alternative)(.+)\} \{(.*)\}$/) {
				#\newcommand{\testGenIntStackmonitorEqualsJCrasherIterationOneTime} {0.16}
				$subject_name = $1;
				$frontend_name = $2;
				$tool_name = $3;
				$iteration_name = $4;
				$measured_item = $5;			
				$measured_result = $6;
				$measured_result =~ s/\*//;
				$new_measured_result = $measured_result;				
				$new_measured_result =~ s/\\%//;
				if ($new_measured_result ne $measured_result) {
					$new_measured_result = $new_measured_result/100.0;
				}
				$testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}{$subject_name} = $new_measured_result;	
			}
		}
	}
}

#Generate the test minimization digrams
#6 total = (Jtest|JCrasher) * (Prtests|Pmmexecs|Time)
foreach $measured_item (keys %testMinData) {
	foreach $tool_name (keys %{$testMinData{$measured_item}}) {
		open(DIAGRAMDATA, ">"."testMinCharts/testMin".$measured_item.$tool_name.".csv") || die "opening diagram data file"."testMinCharts/testMin".$measured_item.$tool_name.".csv"." $!";
		print DIAGRAMDATA " ";
		foreach $subject_name (sort {$subjects{$a} <=> $subjects{$b}} keys %subjects) {
			print DIAGRAMDATA ",".$subjects_show_name{$subject_name};
		}
		print DIAGRAMDATA "\n";						
		foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %{$testMinData{$measured_item}{$tool_name}}) {
			if ($frontend_name eq "monitorWField") {
				next;
			}
			print DIAGRAMDATA $frontends_show_name{$frontend_name};
			foreach $subject_name (sort {$subjects{$a} <=> $subjects{$b}} keys %{$testMinData{$measured_item}{$tool_name}{$frontend_name}}) {
				print DIAGRAMDATA ",".$testMinData{$measured_item}{$tool_name}{$frontend_name}{$subject_name};
			}
			print DIAGRAMDATA "\n";	
		}
		close(DIAGRAMDATA);
	}
}

#Generate the test generation digrams
#4 total = (tests|mexecs|neobjs) by frontends [average on all subjects, putting jtest and jcrasher side by side] + bcov by subjects
%testGenTotal = ();
%testGenCount = ();
%testGenAvg = ();

%om_subjects = ();
foreach $measured_item (keys %testGenData) {
	foreach $tool_name (keys %{$testGenData{$measured_item}}) {
		foreach $iteration_name (keys %{$testGenData{$measured_item}{$tool_name}}) {
			foreach $frontend_name (keys %{$testGenData{$measured_item}{$tool_name}{$iteration_name}}) {
				if ($frontend_name eq "monitorSeq") {
					next;
				}
				if ($frontend_name eq "monitorWField") {
					next;
				}				
				foreach $subject_name (keys %{$testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}}) {
					if (($testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}{$subject_name} eq "om")) {
						$om_subjects{$measured_item}{$tool_name}{$subject_name} = 1;
					} 						
				}
			}
		}
	}
}

foreach $measured_item (keys %testGenData) {
	foreach $tool_name (keys %{$testGenData{$measured_item}}) {
		foreach $iteration_name (keys %{$testGenData{$measured_item}{$tool_name}}) {
			foreach $frontend_name (keys %{$testGenData{$measured_item}{$tool_name}{$iteration_name}}) {
				if ($frontend_name eq "monitorSeq") {
					next;
				}
				if ($frontend_name eq "monitorWField") {
					next;
				}								
				foreach $subject_name (keys %{$testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}}) {
					if (exists $om_subjects{$measured_item}{$tool_name}{$subject_name}) {#we don't count those subjects that contain om in iteration 2
						next;
					}
					if (not ($testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}{$subject_name} eq "om")) {
						if (not (defined $testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name})) {
							$testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} = 0;
							$testGenCount{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} = 0;
						}
						$testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} = $testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} + $testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}{$subject_name};
						$testGenCount{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} = $testGenCount{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} + 1;
					} 						
				}
			}
		}
	}
}
open(DIAGRAMSTATISTICS, ">"."testGenCharts/IgnoreSubjects.txt") || die "opening diagram data file"."testGenCharts/IgnoreSubjects.txt"." $!";
foreach $measured_item (keys %om_subjects) {
	foreach $tool_name (keys %{$om_subjects{$measured_item}}) {
		$count = 0;
		foreach $subject_name (keys %{$om_subjects{$measured_item}{$tool_name}}) {
			$count++;
		}
		print DIAGRAMSTATISTICS "#ignored subjects for ".$measured_item." by ".$tool_name.": ".$count."\n";
		foreach $subject_name (keys %{$om_subjects{$measured_item}{$tool_name}}) {
			print DIAGRAMSTATISTICS $subject_name."\n";
		}
		print DIAGRAMSTATISTICS "\n";
	}
}

foreach $measured_item (keys %testGenTotal) {
	foreach $tool_name (keys %{$testGenTotal{$measured_item}}) {
		foreach $iteration_name (keys %{$testGenTotal{$measured_item}{$tool_name}}) {	
			foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %{$testGenTotal{$measured_item}{$tool_name}{$iteration_name}}) {
				if ($frontend_name eq "monitorWField") {
					next;
				}								
				if ($measured_item eq "Neobjs") {
					$testGenAvg{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} = $testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} * 1.0 / $testGenCount{$measured_item}{$tool_name}{$iteration_name}{$frontend_name};
				} else {
					$testGenAvg{$measured_item}{$tool_name}{$frontend_name}{$iteration_name} = $testGenTotal{$measured_item}{$tool_name}{$iteration_name}{$frontend_name} * 1.0 / $testGenCount{$measured_item}{$tool_name}{$iteration_name}{$frontend_name};
				}
					
			}
		}
	}
}

#the following three average on all subjects
#tests/mexecs: x-axis: orig (jtest-orig, jcrasher-orig), frontends (jtest-iteration2, jcrasher-iteration-2)
foreach $measured_item (keys %testGenAvg) {
	if (not (($measured_item eq "Tests") or ($measured_item eq "Mexecs"))) {
		next;
	}
	open(DIAGRAMDATA, ">"."testGenCharts/testGen".$measured_item.".csv") || die "opening diagram data file"."testGenCharts/testGen".$measured_item.".csv"." $!";
	print DIAGRAMDATA " ";
	print DIAGRAMDATA ",Existing";
	foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %frontends) {
		if ($frontend_name eq "monitorSeq") {
			next;
		}
		if ($frontend_name eq "monitorWField") {
			next;
		}						
		print DIAGRAMDATA ",".$frontends_show_name{$frontend_name};
	}
	print DIAGRAMDATA "\n";
	#  ,Jtest/JCrasher, frontend0, ....frontend5
	foreach $tool_name (keys %{$testGenAvg{$measured_item}}) {
		print DIAGRAMDATA $tool_name;
		$iteration_name = "Orig";
		print DIAGRAMDATA ",".$testGenAvg{$measured_item}{$tool_name}{"pairwiseEquals"}{$iteration_name};
		foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %{$testGenAvg{$measured_item}{$tool_name}}) {
			$iteration_name = "IterationTwo";
			print DIAGRAMDATA ",".$testGenAvg{$measured_item}{$tool_name}{$frontend_name}{$iteration_name};
		}
		print DIAGRAMDATA "\n";
	}
	close(DIAGRAMDATA);
}

#neobjs: x-axis: frontends (jtest-orig, jtest-iter2, jcrasher-orig, jcrasher-iteration2)
foreach $measured_item (keys %testGenAvg) {
	if (not ($measured_item eq "Neobjs")) {
		next;
	}
	open(DIAGRAMDATA, ">"."testGenCharts/testGen".$measured_item.".csv") || die "opening diagram data file"."testGenCharts/testGen".$measured_item.".csv"." $!";
	print DIAGRAMDATA " ";
	foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %frontends) {
		if ($frontend_name eq "monitorSeq") {
			next;
		}
		if ($frontend_name eq "monitorWField") {
			next;
		}						
		print DIAGRAMDATA ",".$frontends_show_name{$frontend_name};
	}
	print DIAGRAMDATA "\n";
	#  ,frontend0, ....frontend5
	foreach $tool_name (keys %{$testGenAvg{$measured_item}}) {
		foreach $iteration_name (sort {$iterations{$a} <=> $iterations{$b}} keys %{$testGenAvg{$measured_item}{$tool_name}}) {
			if ($iteration_name eq "IterationOne") {
				next;
			}
			print DIAGRAMDATA $tool_name."-".$iterations_show_name{$iteration_name};
			foreach $frontend_name (sort {$frontends{$a} <=> $frontends{$b}} keys %{$testGenAvg{$measured_item}{$tool_name}{$iteration_name}}) {
				if ($frontend_name eq "monitorWField") {
					next;
				}								
				print DIAGRAMDATA ",".$testGenAvg{$measured_item}{$tool_name}{$iteration_name}{$frontend_name};
			}
			print DIAGRAMDATA "\n";
		}
	}
	close(DIAGRAMDATA);
}

#the bcov one sample on all techniques
#bcov: x-axis: subject (Jtest-Existing, Jtest-Iteration2, JCrasher-Existing, Jcrasher-Iteration2)
foreach $measured_item (keys %testGenData) {
	if (not ($measured_item eq "BCov")) {
		next;
	}
	open(DIAGRAMDATA, ">"."testGenCharts/testGen".$measured_item.".csv") || die "opening diagram data file"."testGenCharts/testGen".$measured_item.".csv"." $!";
	print DIAGRAMDATA " ";
	foreach $subject_name (sort {$subjects{$a} <=> $subjects{$b}} keys %subjects) {
		print DIAGRAMDATA ",".$subjects_show_name{$subject_name};
	}
	print DIAGRAMDATA "\n";
	
	foreach $tool_name (keys %{$testGenData{$measured_item}}) {
		foreach $iteration_name (sort {$iterations{$a} <=> $iterations{$b}} keys %{$testGenData{$measured_item}{$tool_name}}) {
			if ($iteration_name eq "IterationOne") {
				next;
			}
			print DIAGRAMDATA $tool_name."-".$iterations_show_name{$iteration_name};
			$frontend_name = "pairwiseEquals";
			foreach $subject_name (keys %{$testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}}) {
				print DIAGRAMDATA ",".$testGenData{$measured_item}{$tool_name}{$iteration_name}{$frontend_name}{$subject_name};
			}
			print DIAGRAMDATA "\n";
		}
	}
	close(DIAGRAMDATA);
}

print "Finishing all excel data file generation!\n";
